%global srcname pycryptodomex

Name:           python-%{srcname}
Version:        3.9.3
Release:        1%{?dist}
Summary:        A self-contained cryptographic library for Python
# PyCrypto-based code is public domain, further PyCryptodome contributions are BSD
License:        BSD and Public Domain
URL:            http://www.pycryptodome.org/
Source0:        https://github.com/Legrandin/pycryptodome/archive/v%{version}/%{srcname}-%{version}.tar.gz

# Use external libtomcrypt library
Patch0:         %{name}-3.7.3-use_external_libtomcrypt.patch
# Fix documentation build with Sphinx <= 1.2, especially on EL
# We're only building the manpage, so we don't need mathjax
Patch1:         %{name}-3.9.0-sphinx.patch
# Port pct-speedtest.py to pycryptodomex
Patch3:         %{name}-3.7.3-pycryptodomex-speedtest.patch

BuildRequires:  coreutils
BuildRequires:  gcc
BuildRequires:  grep
BuildRequires:  libtomcrypt-devel
BuildRequires:  python2-devel
BuildRequires:  python2-setuptools
BuildRequires:  sed
# Needed for documentation
BuildRequires:  python-sphinx10
BuildRequires:  python-sphinxcontrib-napoleon

Provides:       python2-pycryptodomex = %{version}-%{release}

%description
PyCryptodome is a self-contained Python package of low-level cryptographic
primitives. It's a fork of PyCrypto. It brings several enhancements with respect
to the last official version of PyCrypto (2.6.1), for instance:

  * Authenticated encryption modes (GCM, CCM, EAX, SIV, OCB)
  * Accelerated AES on Intel platforms via AES-NI
  * Elliptic curves cryptography (NIST P-256 curve only)
  * Better and more compact API (nonce and iv attributes for ciphers, automatic
    generation of random nonces and IVs, simplified CTR cipher mode, and more)
  * SHA-3 (including SHAKE XOFs) and BLAKE2 hash algorithms
  * Salsa20 and ChaCha20 stream ciphers
  * scrypt and HKDF
  * Deterministic (EC)DSA
  * Password-protected PKCS#8 key containers
  * Shamir's Secret Sharing scheme
  * Random numbers get sourced directly from the OS (and not from a CSPRNG in
    userspace)
  * Cleaner RSA and DSA key generation (largely based on FIPS 186-4)
  * Major clean ups and simplification of the code base

PyCryptodome is not a wrapper to a separate C library like OpenSSL. To the
largest possible extent, algorithms are implemented in pure Python. Only the
pieces that are extremely critical to performance (e.g. block ciphers) are
implemented as C extensions.

Note: all modules are installed under the Cryptodome package to avoid conflicts
with the PyCrypto library.

# Don't want provides for python shared objects
%global _use_internal_dependency_generator 0
%global __deploop() while read FILE; do /usr/lib/rpm/rpmdeps -%{1} ${FILE}; done | sort -u
%global __find_provides /bin/sh -c "grep -v '%{python_sitearch}/Cryptodome/[^/]*/.*\\.so$' | %{__deploop P}"
%global __find_requires /bin/sh -c "%{__deploop R}"

%prep
%autosetup -n pycryptodome-%{version} -p0

# Drop bundled libraries
rm -r src/libtom/

# Remove shebang
sed '1{\@^#! /usr/bin/env python@d}' lib/Crypto/SelfTest/__main__.py >lib/Crypto/SelfTest/__main__.py.new && \
  touch -r lib/Crypto/SelfTest/__main__.py lib/Crypto/SelfTest/__main__.py.new && \
  mv lib/Crypto/SelfTest/__main__.py.new lib/Crypto/SelfTest/__main__.py

%build
touch .separate_namespace
CFLAGS="%{optflags}" %{__python} setup.py build

# Build documentation
make -C Doc/ man SPHINXBUILD=/usr/bin/sphinx-1.0-build

%install
%{__python} setup.py install -O1 --skip-build --root %{buildroot}

# Install man page
install -Dpm 0644 Doc/_build/man/pycryptodome.1 %{buildroot}%{_mandir}/man1/pycryptodome.1

# Fix permissions
chmod -c +x %{buildroot}%{python_sitearch}/Cryptodome/SelfTest/PublicKey/test_vectors/ECC/gen_ecc_p{256,384,521}.sh \
            %{buildroot}%{python_sitearch}/Cryptodome/SelfTest/PublicKey/test_vectors/RSA/gen_rsa_2048.sh

%check
%{__python} setup.py test

# Benchmark
PYTHONPATH=%{buildroot}%{python_sitearch} %{__python} pct-speedtest.py

%files
%doc LICENSE.rst
%doc AUTHORS.rst Changelog.rst README.rst
%{python_sitearch}/Cryptodome/
%{python_sitearch}/%{srcname}-*.egg-info/
%{_mandir}/man1/pycryptodome.1.*

%changelog
* Tue Nov 12 2019 Paul Howarth <paul@city-fan.org> - 3.9.3-1
- Update to 3.9.3
  - Align stack of functions using SSE2 intrinsics to avoid crashes when
    compiled with gcc on 32-bit x86 platforms (GH#308)
- Stop building with -mstackrealign for ix86 as it's no longer needed
- Add version and release to provide for python2-pycryptodomex
- Fix permissions of self-test key generation scripts

* Sun Nov 10 2019 Paul Howarth <paul@city-fan.org> - 3.9.2-1
- Update to 3.9.2
  - Minor speed-up when importing RSA (GH#328)
  - Avoid allocating arrays of '__m128i' on the stack, to cope with buggy
    compilers (GH#308)
  - Remove blanket '-O3' optimization for gcc and clang, to cope with buggy
    compilers (GH#322)
  - Fix typing stubs for signatures (GH#337)
  - Deal with gcc installations that don't have 'x86intrin.h' (GH#338)

* Wed Oct  9 2019 Paul Howarth <paul@city-fan.org> - 3.9.0-1
- Rewrite for EPEL-6 (python 2 only) without modern python and other macros
- Update to 3.9.0
- Build with -mstackrealign for ix86 to avoid segfaults
  https://github.com/Legrandin/pycryptodome/issues/308

* Fri Feb 15 2019 Mohamed El Morabity <melmorabity@fedoraproject.org> - 3.7.3-1
- Update to 3.7.3

* Mon Nov 19 2018 Mohamed El Morabity <melmorabity@fedoraproject.org> - 3.7.0-1
- Update to 3.7.0
- Use the same .spec file for all supported releases of Fedora and EL
